#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "rfm01/rf01.h"

#define set_bit(r,b)    r |=  (1 << b)
#define clear_bit(r,b)  r &= ~(1 << b)

#define P_C_SWITCH PD3
#define P_C_LED    PD0
#define P_T_SWITCH PD2
#define P_T_LED    PD1
#define P_T_OUT0   PB0
#define P_T_OUT1   PB1

#define P_MOSI     PB3
#define P_MISO     PB4
#define P_SCK      PB5
#define P_SS       PB2

volatile uint8_t sigT;
volatile uint8_t sigC;

ISR(INT0_vect) {
	//clear_bit(GICR, INT0);
	sigT = 1;
}

ISR(INT1_vect) {
	//clear_bit(GICR, INT1);
	sigC = 1;
}

ISR(SPI_STC_vect) {
	;
}


void SPI_MasterTransmit(char data)
{
	SPDR = data;
	while(!(SPSR & (1<<SPIF)))
		;
}

void blinkLed(uint8_t led)
{
	int i;
	const int ds=10;
	const int ds2=ds/2;
	for(i=0; i<ds2; ++i)
	{
		_delay_ms(10);
	}
	set_bit(PORTD, led);
	for(i=0; i<ds; ++i)
	{
		_delay_ms(10);
	}
	clear_bit(PORTD, led);
	for(i=0; i<ds2; ++i)
	{
		_delay_ms(10);
	}
}

int main(int argc, char **argv) {

	//unsigned char data[3];
	uint8_t SYS_ID = 73;
//	uint8_t id;
	
	// Configure flash-pin as input, led- and flash-port as output
	DDRD |= (0<<P_C_SWITCH) | (0<<P_T_SWITCH) | (1<<P_C_LED) | (1<<P_T_LED);
	DDRB |= (1<<P_T_OUT0) | (1<<P_T_OUT1);

	// SPI
	//DDRB |= (1<<P_MOSI) | (1<<P_SCK) | (1<<P_SS);
	//SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR0);

	// Pull-Up-Resistor must be deactivted for the trigger-input
	set_bit(PORTD, P_T_SWITCH);
	set_bit(PORTD, P_C_SWITCH);

	//Prepare Interrupt-Handling. Falling Edge Interrupt on INTx
	set_bit(MCUCR, ISC11);
	clear_bit(MCUCR, ISC10);
	set_bit(MCUCR, ISC01);
	clear_bit(MCUCR, ISC00);
	
	set_bit(GICR, INT1);
	set_bit(GICR, INT0);

	sigT = 0;
	sigT = 0;

	    
	rf01_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf01_setfreq(RF01FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf01_setbandwidth(4);			// 200kHz Bandbreite
	rf01_setreceiver(2,4);			// -6dB Verst�rkung, DRSSI threshold: -79dBm 
//	rf01_setbaud(19200);			// 19200 Baud
	rf01_setbaud(38400);			// 19200 Baud
//	rf01_setbaud(76800);			// 19200 Baud

	blinkLed(P_T_LED);
   	blinkLed(P_C_LED);
  	blinkLed(P_C_LED);

	while(1)
	{
		set_bit(PORTD, P_C_LED);
		if(SYS_ID == rf01_rxTCmd())
		{
			set_bit(PORTB, P_T_OUT0);
			set_bit(PORTB, P_T_OUT1);
			_delay_ms(20);
			clear_bit(PORTB, P_T_OUT0);
			clear_bit(PORTB, P_T_OUT1);
			blinkLed(P_T_LED);
		}
		else
		{
			clear_bit(PORTD, P_C_LED);
			blinkLed(P_C_LED);
		}
	}	
}
