#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <avr/sleep.h>
#include "rfm02/global.h"
#include "rfm02/rf02.h"

//#define F_CPU 10000000UL
#include <util/delay.h>

#define set_bit(r,b)    r |=  (1 << b)
#define clear_bit(r,b)  r &= ~(1 << b)

#define P_C_LED    PD0
#define P_T_LED    PD1
#define P_T_SWITCH PD2
#define P_C_SWITCH PD3

volatile uint8_t sigT;

void blinkLed(uint8_t led)
{
	int i;
	const int ds=10;
	const int ds2=ds/2;
	for(i=0; i<ds2; ++i)
	{
		_delay_ms(10);
	}
	sbi(PORTD, led);
	for(i=0; i<ds; ++i)
	{
		_delay_ms(10);
	}
	cbi(PORTD, led);
	for(i=0; i<ds2; ++i)
	{
		_delay_ms(10);
	}
}

ISR(INT0_vect) {
	sigT = 1;
}


int main(void)
{
	DDRD = (1<<P_C_LED) | (1<<P_T_LED);
	set_bit(PORTD, P_T_SWITCH);

	set_bit(MCUCR, ISC11);
	clear_bit(MCUCR, ISC10);
	set_bit(MCUCR, ISC01);
	clear_bit(MCUCR, ISC00);
	set_bit(GICR, INT0);
	sigT = 0;
	
//	blinkLed(P_C_LED);
//	blinkLed(P_C_LED);

	rf02_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf02_setfreq(RF02FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf02_setpower(4);				// -12dBm Ausgangangsleistung
	rf02_setmodfreq(3);				// 120kHz Frequenzshift
//	rf02_setbaud(19200);			// 19200 Baud
	rf02_setbaud(38400);			// 19200 Baud
//	rf02_setbaud(56000);			// 19200 Baud

	blinkLed(P_T_LED);
	blinkLed(P_C_LED);
	blinkLed(P_C_LED);

	sbi(PORTD, P_C_LED);
	
	for (;;)
	{
		sei();

		if((sigT == 0)){
			set_sleep_mode(SLEEP_MODE_IDLE);
			sleep_mode();
		}

		cli();
		if(sigT){
			rf02_txTCmd(73);
			blinkLed(P_T_LED);
			sigT = 0;
		}

	}
}

